@@ -22,8 +22,11 @@
 #include <string.h>
 #include <assert.h>
 
-#include "mpvcore/m_option.h"
-#include "mpvcore/m_config.h"
+#include "common/common.h"
+#include "common/global.h"
+
+#include "options/m_option.h"
+#include "options/m_config.h"
 
 #include "af.h"
 
@@ -52,18 +55,20 @@ extern struct af_info af_info_karaoke;
 extern struct af_info af_info_scaletempo;
 extern struct af_info af_info_bs2b;
 extern struct af_info af_info_lavfi;
+extern struct af_info af_info_convert24;
+extern struct af_info af_info_convertsignendian;
 
 static struct af_info* filter_list[] = {
     &af_info_dummy,
     &af_info_delay,
     &af_info_channels,
-    &af_info_force,
+    &af_info_format,
     &af_info_volume,
     &af_info_equalizer,
     &af_info_pan,
     &af_info_surround,
     &af_info_sub,
-#ifdef HAVE_SYS_MMAN_H
+#if HAVE_SYS_MMAN_H
     &af_info_export,
 #endif
     &af_info_drc,
@@ -72,21 +77,22 @@ static struct af_info* filter_list[] = {
     &af_info_lavrresample,
     &af_info_sweep,
     &af_info_hrtf,
-#ifdef CONFIG_LADSPA
+#if HAVE_LADSPA
     &af_info_ladspa,
 #endif
     &af_info_center,
     &af_info_sinesuppress,
     &af_info_karaoke,
     &af_info_scaletempo,
-#ifdef CONFIG_LIBBS2B
+#if HAVE_LIBBS2B
     &af_info_bs2b,
 #endif
-#ifdef CONFIG_AF_LAVFI
+#if HAVE_AF_LAVFI
     &af_info_lavfi,
 #endif
-    // Must come last, because it's the fallback format conversion filter
-    &af_info_format,
+    // Must come last, because they're fallback format conversion filter
+    &af_info_convert24,
+    &af_info_convertsignendian,
     NULL
 };
 
@@ -109,7 +115,10 @@ static bool get_desc(struct m_obj_desc *dst, int index)
 const struct m_obj_list af_obj_list = {
     .get_desc = get_desc,
     .description = "audio filters",
-    .legacy_hacks = true, // many filters have custom option parsing
+    .aliases = {
+        {"force",     "format"},
+        {0}
+    },
 };
 
 static bool af_config_equals(struct mp_audio *a, struct mp_audio *b)
@@ -133,7 +142,7 @@ static int input_control(struct af_instance* af, int cmd, void* arg)
 {
     switch (cmd) {
     case AF_CONTROL_REINIT:
-        assert(arg == &((struct af_stream *)af->setup)->input);
+        assert(arg == &((struct af_stream *)af->priv)->input);
         return AF_OK;
     }
     return AF_UNKNOWN;
@@ -141,7 +150,7 @@ static int input_control(struct af_instance* af, int cmd, void* arg)
 
 static int output_control(struct af_instance* af, int cmd, void* arg)
 {
-    struct af_stream *s = af->setup;
+    struct af_stream *s = af->priv;
     struct mp_audio *output = &s->output;
     struct mp_audio *filter_output = &s->filter_output;
 
@@ -159,9 +168,9 @@ static int output_control(struct af_instance* af, int cmd, void* arg)
     return AF_UNKNOWN;
 }
 
-static struct mp_audio *dummy_play(struct af_instance* af, struct mp_audio* data)
+static int dummy_filter(struct af_instance* af, struct mp_audio* data, int f)
 {
-    return data;
+    return 0;
 }
 
 /* Function for creating a new filter of type name.The name may
@@ -171,8 +180,7 @@ static struct af_instance *af_create(struct af_stream *s, char *name,
 {
     struct m_obj_desc desc;
     if (!m_obj_list_find(&desc, &af_obj_list, bstr0(name))) {
-        mp_tmsg(MSGT_VFILTER, MSGL_ERR,
-                "Couldn't find audio filter '%s'.\n", name);
+        MP_ERR(s, "Couldn't find audio filter '%s'.\n", name);
         return NULL;
     }
     const struct af_info *info = desc.p;
@@ -181,40 +189,37 @@ static struct af_instance *af_create(struct af_stream *s, char *name,
     if (info->flags & AF_FLAGS_NOT_REENTRANT) {
         for (struct af_instance *cur = s->first; cur; cur = cur->next) {
             if (cur->info == info) {
-                mp_msg(MSGT_AFILTER, MSGL_ERR, "[libaf] There can only be one "
+                MP_ERR(s, "There can only be one "
                        "instance of the filter '%s' in each stream\n", name);
                 return NULL;
             }
         }
     }
 
-    mp_msg(MSGT_AFILTER, MSGL_V, "[libaf] Adding filter %s \n", name);
+    MP_VERBOSE(s, "Adding filter %s \n", name);
 
     struct af_instance *af = talloc_zero(NULL, struct af_instance);
     *af = (struct af_instance) {
         .info = info,
         .mul = 1,
+        .data = talloc_zero(af, struct mp_audio),
+        .log = mp_log_new(af, s->log, name),
     };
-    struct m_config *config = m_config_from_obj_desc(af, &desc);
-    if (m_config_initialize_obj(config, &desc, &af->priv, &args) < 0)
+    struct m_config *config = m_config_from_obj_desc(af, s->log, &desc);
+    if (m_config_apply_defaults(config, name, s->opts->af_defs) < 0)
         goto error;
+    if (m_config_set_obj_params(config, args) < 0)
+        goto error;
+    af->priv = config->optstruct;
 
     // Initialize the new filter
     if (af->info->open(af) != AF_OK)
         goto error;
-    if (args && af->control) {
-        // Single option string for old filters
-        char *opts = (char *)args; // m_config_initialize_obj did this
-        assert(!af->priv);
-        if (af->control(af, AF_CONTROL_COMMAND_LINE, opts) <= AF_ERROR)
-            goto error;
-    }
 
     return af;
 
 error:
-    mp_msg(MSGT_AFILTER, MSGL_ERR,
-           "[libaf] Couldn't create or open audio filter '%s'\n", name);
+    MP_ERR(s, "Couldn't create or open audio filter '%s'\n", name);
     talloc_free(af);
     return NULL;
 }
@@ -275,17 +280,14 @@ static void af_remove(struct af_stream *s, struct af_instance *af)
         return;
 
     // Print friendly message
-    mp_msg(MSGT_AFILTER, MSGL_V, "[libaf] Removing filter %s \n",
-           af->info->name);
-
-    // Notify filter before changing anything
-    af->control(af, AF_CONTROL_PRE_DESTROY, 0);
+    MP_VERBOSE(s, "Removing filter %s \n", af->info->name);
 
     // Detach pointers
     af->prev->next = af->next;
     af->next->prev = af->prev;
 
-    af->uninit(af);
+    if (af->uninit)
+        af->uninit(af);
     talloc_free(af);
 }
 
@@ -303,26 +305,26 @@ repeat:
 static void af_print_filter_chain(struct af_stream *s, struct af_instance *at,
                                   int msg_level)
 {
-    mp_msg(MSGT_AFILTER, msg_level, "Audio filter chain:\n");
+    MP_MSG(s, msg_level, "Audio filter chain:\n");
 
     struct af_instance *af = s->first;
     while (af) {
-        mp_msg(MSGT_AFILTER, msg_level, "  [%s] ", af->info->name);
+        MP_MSG(s, msg_level, "  [%s] ", af->info->name);
         if (af->data) {
             char *info = mp_audio_config_to_str(af->data);
-            mp_msg(MSGT_AFILTER, msg_level, "%s", info);
+            MP_MSG(s, msg_level, "%s", info);
             talloc_free(info);
         }
         if (af == at)
-            mp_msg(MSGT_AFILTER, msg_level, " <-");
-        mp_msg(MSGT_AFILTER, msg_level, "\n");
+            MP_MSG(s, msg_level, " <-");
+        MP_MSG(s, msg_level, "\n");
 
         af = af->next;
     }
 
-    mp_msg(MSGT_AFILTER, msg_level, "  [ao] ");
+    MP_MSG(s, msg_level, "  [ao] ");
     char *info = mp_audio_config_to_str(&s->output);
-    mp_msg(MSGT_AFILTER, msg_level, "%s\n", info);
+    MP_MSG(s, msg_level, "%s\n", info);
     talloc_free(info);
 }
 
@@ -334,14 +336,76 @@ static int af_count_filters(struct af_stream *s)
     return count;
 }
 
-static char *af_find_conversion_filter(int srcfmt, int dstfmt)
+// Finds the first conversion filter on the way from srcfmt to dstfmt.
+// Conversions form a DAG: each node is a format/filter pair, and possible
+// conversions are edges. We search the DAG for the shortest path.
+// Some cases visit the same filter multiple times, but with different formats
+// (like u24le->s8), so one node per format or filter separate is not enough.
+// Returns the filter and dest. format for the first conversion step.
+// (So we know what conversion filter with what format to insert next.)
+static char *af_find_conversion_filter(int srcfmt, int *dstfmt)
 {
-    for (int n = 0; filter_list[n]; n++) {
-        struct af_info *af = filter_list[n];
-        if (af->test_conversion && af->test_conversion(srcfmt, dstfmt))
-            return (char *)af->name;
+#define NUM_FMT 64
+#define NUM_FILT 32
+#define NUM_NODES (NUM_FMT * NUM_FILT)
+    for (int n = 0; filter_list[n]; n++)
+        assert(n < NUM_FILT);
+    for (int n = 0; af_fmtstr_table[n].format; n++)
+        assert(n < NUM_FMT);
+
+    bool visited[NUM_NODES] = {0};
+    unsigned char distance[NUM_NODES];
+    short previous[NUM_NODES] = {0};
+    for (int n = 0; n < NUM_NODES; n++) {
+        distance[n] = 255;
+        if (af_fmtstr_table[n % NUM_FMT].format == srcfmt)
+            distance[n] = 0;
     }
-    return NULL;
+
+    while (1) {
+        int next = -1;
+        for (int n = 0; n < NUM_NODES; n++) {
+            if (!visited[n] && (next < 0 || (distance[n] < distance[next])))
+                next = n;
+        }
+        if (next < 0 || distance[next] == 255)
+            return NULL;
+        visited[next] = true;
+
+        int fmt = next % NUM_FMT;
+        if (af_fmtstr_table[fmt].format == *dstfmt) {
+            // Best match found
+            for (int cur = next; cur >= 0; cur = previous[cur] - 1) {
+                if (distance[cur] == 1) {
+                    *dstfmt = af_fmtstr_table[cur % NUM_FMT].format;
+                    return (char *)filter_list[cur / NUM_FMT]->name;
+                }
+            }
+            return NULL;
+        }
+
+        for (int n = 0; filter_list[n]; n++) {
+            struct af_info *af = filter_list[n];
+            if (!af->test_conversion)
+                continue;
+            for (int i = 0; af_fmtstr_table[i].format; i++) {
+                if (i != fmt && af->test_conversion(af_fmtstr_table[fmt].format,
+                                                    af_fmtstr_table[i].format))
+                {
+                    int other = n * NUM_FMT + i;
+                    int ndist = distance[next] + 1;
+                    if (ndist < distance[other]) {
+                        distance[other] = ndist;
+                        previous[other] = next + 1;
+                    }
+                }
+            }
+        }
+    }
+    assert(0);
+#undef NUM_FMT
+#undef NUM_FILT
+#undef NODE_N
 }
 
 static bool af_is_conversion_filter(struct af_instance *af)
@@ -352,7 +416,7 @@ static bool af_is_conversion_filter(struct af_instance *af)
 // in is what af can take as input - insert a conversion filter if the actual
 // input format doesn't match what af expects.
 // Returns:
-//   AF_OK: must call af_reinit() or equivalent, format matches
+//   AF_OK: must call af_reinit() or equivalent, format matches (or is closer)
 //   AF_FALSE: nothing was changed, format matches
 //   else: error
 static int af_fix_format_conversion(struct af_stream *s,
@@ -365,18 +429,21 @@ static int af_fix_format_conversion(struct af_stream *s,
     struct mp_audio actual = *prev->data;
     if (actual.format == in.format)
         return AF_FALSE;
-    if (prev->control(prev, AF_CONTROL_FORMAT_FMT, &in.format) == AF_OK) {
-        *p_af = prev;
-        return AF_OK;
-    }
-    char *filter = af_find_conversion_filter(actual.format, in.format);
+    int dstfmt = in.format;
+    char *filter = af_find_conversion_filter(actual.format, &dstfmt);
     if (!filter)
         return AF_ERROR;
+    if (strcmp(filter, prev->info->name) == 0) {
+        if (prev->control(prev, AF_CONTROL_SET_FORMAT, &dstfmt) == AF_OK) {
+            *p_af = prev;
+            return AF_OK;
+        }
+    }
     struct af_instance *new = af_prepend(s, af, filter, NULL);
     if (new == NULL)
         return AF_ERROR;
     new->auto_inserted = true;
-    if (AF_OK != (rv = new->control(new, AF_CONTROL_FORMAT_FMT, &in.format)))
+    if (AF_OK != (rv = new->control(new, AF_CONTROL_SET_FORMAT, &dstfmt)))
         return rv;
     *p_af = new;
     return AF_OK;
@@ -392,7 +459,7 @@ static int af_fix_channels(struct af_stream *s, struct af_instance **p_af,
     struct mp_audio actual = *prev->data;
     if (mp_chmap_equals(&actual.channels, &in.channels))
         return AF_FALSE;
-    if (prev->control(prev, AF_CONTROL_CHANNELS, &in.channels) == AF_OK) {
+    if (prev->control(prev, AF_CONTROL_SET_CHANNELS, &in.channels) == AF_OK) {
         *p_af = prev;
         return AF_OK;
     }
@@ -401,7 +468,7 @@ static int af_fix_channels(struct af_stream *s, struct af_instance **p_af,
     if (new == NULL)
         return AF_ERROR;
     new->auto_inserted = true;
-    if (AF_OK != (rv = new->control(new, AF_CONTROL_CHANNELS, &in.channels)))
+    if (AF_OK != (rv = new->control(new, AF_CONTROL_SET_CHANNELS, &in.channels)))
         return rv;
     *p_af = new;
     return AF_OK;
@@ -416,7 +483,7 @@ static int af_fix_rate(struct af_stream *s, struct af_instance **p_af,
     struct mp_audio actual = *prev->data;
     if (actual.rate == in.rate)
         return AF_FALSE;
-    if (prev->control(prev, AF_CONTROL_RESAMPLE_RATE, &in.rate) == AF_OK) {
+    if (prev->control(prev, AF_CONTROL_SET_RESAMPLE_RATE, &in.rate) == AF_OK) {
         *p_af = prev;
         return AF_OK;
     }
@@ -425,7 +492,7 @@ static int af_fix_rate(struct af_stream *s, struct af_instance **p_af,
     if (new == NULL)
         return AF_ERROR;
     new->auto_inserted = true;
-    if (AF_OK != (rv = new->control(new, AF_CONTROL_RESAMPLE_RATE, &in.rate)))
+    if (AF_OK != (rv = new->control(new, AF_CONTROL_SET_RESAMPLE_RATE, &in.rate)))
         return rv;
     *p_af = new;
     return AF_OK;
@@ -439,10 +506,12 @@ static int af_fix_rate(struct af_stream *s, struct af_instance **p_af,
 // In that case, you should always rebuild the filter chain, or abort.
 static int af_reinit(struct af_stream *s)
 {
+    remove_auto_inserted_filters(s);
     // Start with the second filter, as the first filter is the special input
     // filter which needs no initialization.
     struct af_instance *af = s->first->next;
-    int max_retry = af_count_filters(s) * 4; // up to 4 retries per filter
+    // Up to 7 retries per filter (channel, rate, 5x format conversions)
+    int max_retry = af_count_filters(s) * 7;
     int retry = 0;
     while (af) {
         if (retry >= max_retry)
@@ -451,10 +520,11 @@ static int af_reinit(struct af_stream *s)
         // Check if this is the first filter
         struct mp_audio in = *af->prev->data;
         // Reset just in case...
-        in.audio = NULL;
-        in.len = 0;
+        mp_audio_set_null_data(&in);
 
         int rv = af->control(af, AF_CONTROL_REINIT, &in);
+        if (rv == AF_OK && !mp_audio_config_equals(&in, af->prev->data))
+            rv = AF_FALSE; // conversion filter needed
         switch (rv) {
         case AF_OK:
             af = af->next;
@@ -484,8 +554,8 @@ static int af_reinit(struct af_stream *s)
             break;
         }
         default:
-            mp_msg(MSGT_AFILTER, MSGL_ERR, "[libaf] Reinitialization did not "
-                   "work, audio filter '%s' returned error code %i\n",
+            MP_ERR(s, "Reinitialization did not work, "
+                   "audio filter '%s' returned error code %i\n",
                    af->info->name, rv);
             af_print_filter_chain(s, af, MSGL_ERR);
             return AF_ERROR;
@@ -502,8 +572,7 @@ static int af_reinit(struct af_stream *s)
     return af_config_equals(&s->output, &s->filter_output) ? AF_OK : AF_ERROR;
 
 negotiate_error:
-    mp_msg(MSGT_AFILTER, MSGL_ERR, "[libaf] Unable to correct audio format. "
-           "This error should never occur, please send a bug report.\n");
+    MP_ERR(s, "Unable to convert audio input format to output format.\n");
     af_print_filter_chain(s, af, MSGL_ERR);
     return AF_ERROR;
 }
@@ -515,7 +584,7 @@ void af_uninit(struct af_stream *s)
         af_remove(s, s->first->next);
 }
 
-struct af_stream *af_new(struct MPOpts *opts)
+struct af_stream *af_new(struct mpv_global *global)
 {
     struct af_stream *s = talloc_zero(NULL, struct af_stream);
     static struct af_info in = { .name = "in" };
@@ -523,8 +592,8 @@ struct af_stream *af_new(struct MPOpts *opts)
     *s->first = (struct af_instance) {
         .info = &in,
         .control = input_control,
-        .play = dummy_play,
-        .setup = s,
+        .filter = dummy_filter,
+        .priv = s,
         .data = &s->input,
         .mul = 1.0,
     };
@@ -533,14 +602,15 @@ struct af_stream *af_new(struct MPOpts *opts)
     *s->last = (struct af_instance) {
         .info = &out,
         .control = output_control,
-        .play = dummy_play,
-        .setup = s,
+        .filter = dummy_filter,
+        .priv = s,
         .data = &s->filter_output,
         .mul = 1.0,
     };
     s->first->next = s->last;
     s->last->prev = s->first;
-    s->opts = opts;
+    s->opts = global->opts;
+    s->log = mp_log_new(s, global->log, "!af");
     return s;
 }
 
@@ -553,7 +623,7 @@ void af_destroy(struct af_stream *s)
 /* Initialize the stream "s". This function creates a new filter list
    if necessary according to the values set in input and output. Input
    and output should contain the format of the current movie and the
-   formate of the preferred output respectively. The function is
+   format of the preferred output respectively. The function is
    reentrant i.e. if called with an already initialized stream the
    stream will be reinitialized.
    If one of the prefered output parameters is 0 the one that needs
@@ -566,8 +636,8 @@ int af_init(struct af_stream *s)
         return -1;
 
     // Precaution in case caller is misbehaving
-    s->input.audio  = s->output.audio  = NULL;
-    s->input.len    = s->output.len    = 0;
+    mp_audio_set_null_data(&s->input);
+    mp_audio_set_null_data(&s->output);
 
     // Check if this is the first call
     if (s->first->next == s->last) {
@@ -581,11 +651,9 @@ int af_init(struct af_stream *s)
         }
     }
 
-    remove_auto_inserted_filters(s);
-
     if (af_reinit(s) != AF_OK) {
         // Something is stuffed audio out will not work
-        mp_msg(MSGT_AFILTER, MSGL_ERR, "Could not create audio filter chain.\n");
+        MP_ERR(s, "Could not create audio filter chain.\n");
         af_uninit(s);
         return -1;
     }
@@ -619,36 +687,31 @@ struct af_instance *af_add(struct af_stream *s, char *name, char **args)
     return new;
 }
 
-// Filter data chunk through the filters in the list
-struct mp_audio *af_play(struct af_stream *s, struct mp_audio *data)
+/* Filter data chunk through the filters in the list.
+ * On success, *data is set to the filtered data/format.
+ * Warning: input audio data will be overwritten.
+ */
+int af_filter(struct af_stream *s, struct mp_audio *data, int flags)
 {
     struct af_instance *af = s->first;
+    assert(mp_audio_config_equals(af->data, data));
     // Iterate through all filters
-    do {
-        if (data->len <= 0)
-            break;
-        data = af->play(af, data);
+    while (af) {
+        int r = af->filter(af, data, flags);
+        if (r < 0)
+            return r;
+        assert(mp_audio_config_equals(af->data, data));
         af = af->next;
-    } while (af && data);
-    return data;
+    }
+    return 0;
 }
 
-/* Calculate the minimum output buffer size for given input data d
- * when using the RESIZE_LOCAL_BUFFER macro. The +t+1 part ensures the
- * value is >= len*mul rounded upwards to whole samples even if the
- * double 'mul' is inexact. */
-int af_lencalc(double mul, struct mp_audio *d)
-{
-    int t = d->bps * d->nch;
-    return d->len * mul + t + 1;
-}
-
-// Calculate average ratio of filter output size to input size
+// Calculate average ratio of filter output samples to input samples.
+// e.g: num_output_samples = mul * num_input_samples
 double af_calc_filter_multiplier(struct af_stream *s)
 {
     struct af_instance *af = s->first;
     double mul = 1;
-    // Iterate through all filters and calculate total multiplication factor
     do {
         mul *= af->mul;
         af = af->next;
@@ -657,41 +720,20 @@ double af_calc_filter_multiplier(struct af_stream *s)
     return mul;
 }
 
-/* Calculate the total delay [bytes output] caused by the filters */
+/* Calculate the total delay [seconds of output] caused by the filters */
 double af_calc_delay(struct af_stream *s)
 {
     struct af_instance *af = s->first;
-    register double delay = 0.0;
-    // Iterate through all filters
+    double delay = 0.0;
     while (af) {
         delay += af->delay;
-        delay *= af->mul;
         af = af->next;
     }
     return delay;
 }
 
-/* Helper function called by the macro with the same name this
-   function should not be called directly */
-int af_resize_local_buffer(struct af_instance *af, struct mp_audio *data)
-{
-    // Calculate new length
-    register int len = af_lencalc(af->mul, data);
-    mp_msg(MSGT_AFILTER, MSGL_V, "[libaf] Reallocating memory in module %s, "
-           "old len = %i, new len = %i\n", af->info->name, af->data->len, len);
-    // If there is a buffer free it
-    free(af->data->audio);
-    // Create new buffer and check that it is OK
-    af->data->audio = malloc(len);
-    if (!af->data->audio) {
-        mp_msg(MSGT_AFILTER, MSGL_FATAL, "[libaf] Could not allocate memory \n");
-        return AF_ERROR;
-    }
-    af->data->len = len;
-    return AF_OK;
-}
-
-// documentation in af.h
+/* Send control to all filters, starting with the last until one accepts the
+ * command with AF_OK. Return the accepting filter. */
 struct af_instance *af_control_any_rev(struct af_stream *s, int cmd, void *arg)
 {
     int res = AF_UNKNOWN;
@@ -704,3 +746,10 @@ struct af_instance *af_control_any_rev(struct af_stream *s, int cmd, void *arg)
     }
     return NULL;
 }
+
+/* Send control to all filters. Never stop, even if a filter returns AF_OK. */
+void af_control_all(struct af_stream *s, int cmd, void *arg)
+{
+    for (struct af_instance *af = s->first; af; af = af->next)
+        af->control(af, cmd, arg);
+}
